@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: video_out_vidix.c,v 1.68 2005/01/23 23:01:12 jstembridge Exp $
+ * $Id: video_out_vidix.c,v 1.75 2006/12/19 19:10:51 dsalt Exp $
  * 
  * video_out_vidix.c
  *
@@ -62,7 +62,6 @@
 
 #include "video_out.h"
 #include "xine_internal.h"
-#include "alphablend.h"
 #include "xineutils.h"
 #include "vo_scale.h"
 
@@ -344,6 +343,7 @@ static uint32_t vidix_get_capabilities (vo_driver_t *this_gen) {
   return this->capabilities;
 }
 
+#ifdef HAVE_FB
 static void vidixfb_frame_output_cb(void *user_data, int video_width, int video_height, double video_pixel_aspect, int *dest_x, int *dest_y, int *dest_width, int *dest_height, double *dest_pixel_aspect, int *win_x, int *win_y) {
   vidix_driver_t *this = (vidix_driver_t *) user_data;
 
@@ -355,6 +355,7 @@ static void vidixfb_frame_output_cb(void *user_data, int video_width, int video_
   *win_x             = 0;
   *win_y             = 0;
 }
+#endif
 
 static void vidix_frame_field (vo_frame_t *vo_img, int which_field) {
   /* not needed for vidix */
@@ -568,9 +569,9 @@ static void vidix_update_frame_format (vo_driver_t *this_gen,
 
 static void vidix_overlay_begin (vo_driver_t *this_gen, 
 			      vo_frame_t *frame_gen, int changed) {
-#ifdef HAVE_X11
   vidix_driver_t  *this = (vidix_driver_t *) this_gen;
 
+#ifdef HAVE_X11
   this->ovl_changed += changed;
 
   if( this->ovl_changed && this->xoverlay ) {
@@ -579,6 +580,9 @@ static void vidix_overlay_begin (vo_driver_t *this_gen,
     XUnlockDisplay (this->display);
   }
 #endif
+  
+  this->alphablend_extra_data.offset_x = frame_gen->overlay_offset_x;
+  this->alphablend_extra_data.offset_y = frame_gen->overlay_offset_y;
 }
 
 static void vidix_overlay_end (vo_driver_t *this_gen, vo_frame_t *vo_img) {
@@ -614,9 +618,9 @@ static void vidix_overlay_blend (vo_driver_t *this_gen, vo_frame_t *frame_gen, v
 #endif
     } else { 
       if( frame->format == XINE_IMGFMT_YV12 )
-        blend_yuv( frame->vo_frame.base, overlay, frame->width, frame->height, frame->vo_frame.pitches, &this->alphablend_extra_data);
+        _x_blend_yuv( frame->vo_frame.base, overlay, frame->width, frame->height, frame->vo_frame.pitches, &this->alphablend_extra_data);
       else
-        blend_yuy2( frame->vo_frame.base[0], overlay, frame->width, frame->height, frame->vo_frame.pitches[0], &this->alphablend_extra_data);
+        _x_blend_yuy2( frame->vo_frame.base[0], overlay, frame->width, frame->height, frame->vo_frame.pitches[0], &this->alphablend_extra_data);
     }
   }
 }
@@ -1214,7 +1218,7 @@ static void *vidix_init_class (xine_t *xine, void *visual_gen) {
   return this;
 }
 
-static vo_info_t vo_info_vidix = {
+static const vo_info_t vo_info_vidix = {
   2,                    /* priority    */
   XINE_VISUAL_TYPE_X11  /* visual type */
 };
@@ -1231,7 +1235,7 @@ static vo_driver_t *vidixfb_open_plugin (video_driver_class_t *class_gen, const
   this->visual_type = XINE_VISUAL_TYPE_FB;
   
   /* Register config option for fb device */
-  device = config->register_string(config, "video.device.vidixfb_device", "/dev/fb0",
+  device = config->register_filename(config, "video.device.vidixfb_device", "/dev/fb0", XINE_CONFIG_STRING_IS_DEVICE_NAME,
     _("framebuffer device name"),
     _("Specifies the file name for the framebuffer device to be used.\n"
       "This setting is security critical, because when changed to a different file, xine "
@@ -1296,7 +1300,7 @@ static void *vidixfb_init_class (xine_t *xine, void *visual_gen) {
   return this;
 }
 
-static vo_info_t vo_info_vidixfb = {
+static const vo_info_t vo_info_vidixfb = {
   2,                    /* priority    */
   XINE_VISUAL_TYPE_FB   /* visual type */
 };
@@ -1306,13 +1310,13 @@ static vo_info_t vo_info_vidixfb = {
  * exported plugin catalog entry
  */
 
-plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */  
 #ifdef HAVE_X11
-  { PLUGIN_VIDEO_OUT, 20, "vidix", XINE_VERSION_CODE, &vo_info_vidix, vidix_init_class },
+  { PLUGIN_VIDEO_OUT, 21, "vidix", XINE_VERSION_CODE, &vo_info_vidix, vidix_init_class },
 #endif
 #ifdef HAVE_FB
-  { PLUGIN_VIDEO_OUT, 20, "vidixfb", XINE_VERSION_CODE, &vo_info_vidixfb, vidixfb_init_class },
+  { PLUGIN_VIDEO_OUT, 21, "vidixfb", XINE_VERSION_CODE, &vo_info_vidixfb, vidixfb_init_class },
 #endif
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
